'use strict';
const fs = require('fs');
const path = require('path');
const nunjucks = require('nunjucks');


function HtmlWebpackJinjaCompilePlugin(options) {
    this.initConfig(options);
}

HtmlWebpackJinjaCompilePlugin.prototype.apply = function(compiler) {
    const self = this;
    compiler.plugin('compilation', function(compilation) {
        compilation.plugin('html-webpack-plugin-after-emit', function(htmlPluginData, callback) {
            if (!htmlPluginData.plugin.options.isJinja) {
                callback(null);
                return;
            }
            const options = htmlPluginData.plugin.options;
            const htmlName = options.htmlName;
            console.assert(htmlName, 'HtmlWebpackJinjaCompilePlugin: missing viewDir/mockDir in options !');
            try {
                const mockJsName = htmlName.replace('.html', '.js');
                const mockFileName = htmlName;
                const mockDir = self.config.mockDir;
                const mockJs = require(path.join(mockDir, mockJsName));
                const res = self.nunjucks.renderString(htmlPluginData.html.source(), mockJs.getTemplateData());
                fs.writeFile(path.join(mockDir,  mockFileName), res, function (err) {
                    if (err) {
                        console.trace(err);
                        return callback(err);
                    }
                    callback(null);
                });
            } catch (err) {
                console.trace(err);
                callback(err);
            }
        });
    });
};

HtmlWebpackJinjaCompilePlugin.prototype.initConfig = function(options) {
    this.config = Object.assign({}, options);
    const viewDir = this.config.viewDir;
    const mockDir = this.config.mockDir;
    console.assert(viewDir && mockDir, 'HtmlWebpackJinjaCompilePlugin: missing viewDir/mockDir in options !');
    this.nunjucks = nunjucks;
    this.env = this.nunjucks.configure(viewDir, { // 设置模板文件的目录
        autoescape: false
    });
    this.nunjucks.installJinjaCompat();
    this.env.addFilter('map', function(name, opts) {
        if (!name) {
            return [];
        }
        return [];
    });

};


module.exports = HtmlWebpackJinjaCompilePlugin;
